-------------------------------------------------------------
-- Mcc.Parallel
--
-- Parallel port interface in Ada using a VXD from Net 2000 Ltd
-- (PortIO 95).  Works only on Win 95/98 (not Windows NT).
-- See http://ds.dial.pipex.com/town/close/ec63/index.html
-- for more information
--
-- By: Dr. Martin C. Carlisle
--     United States Air Force Academy
--     Department of Computer Science
--
-- This is free software; you can redistribute it and/or 
-- modify without restriction.  We do ask that you please keep
-- the original author information, and clearly indicate if the
-- software has been modified.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
-------------------------------------------------------------

package Mcc.Parallel is
    subtype Parallel_Port_Number is Natural range 1..4;
    type Byte is mod 256; -- an 8 bit number
    type Nibble is mod 16; -- a 4 bit number

    type Writeable_Pin is (
       PIN1, PIN2, PIN3, PIN4, PIN5, PIN6, PIN7, PIN8, PIN9,
       PIN14, PIN16, PIN17);
    for Writeable_Pin use (
       PIN1  => 1,  PIN2  => 2, PIN3 => 3, PIN4 => 4, PIN5  => 5,
       PIN6  => 6,  PIN7  => 7, PIN8 => 8, PIN9 => 9, PIN14 => 14,
       PIN16 => 16, PIN17 => 17);

    type Readable_Pin is (
       PIN1, PIN11, PIN12, PIN13, PIN14, PIN15, PIN16, PIN17);
    for Readable_Pin use (
       PIN1  => 1,  PIN11 => 11, PIN12 => 12, PIN13 => 13, PIN14 => 14,
       PIN15 => 15, PIN16 => 16, PIN17 => 17);

   ---------------------------------------------------------------
   -- function(s) Is_Control_Pin
   --
   -- return true if PIN is 1,14,16,17, false o/w
   ---------------------------------------------------------------
   function Is_Control_Pin(Pin : in Readable_Pin) return Boolean;
   function Is_Control_Pin(Pin : in Writeable_Pin) return Boolean;


   ---------------------------------------------------------------
   -- procedure Initialize_Parallel_Port
   --
   -- Must be called first to set up the parallel port (1-4)
   ---------------------------------------------------------------
    procedure Initialize_Parallel_Port(
       Port_Number : in Parallel_Port_Number);


   ---------------------------------------------------------------
   -- procedure Unload_Parallel_Port
   --
   -- Call to remove virtual device
   ---------------------------------------------------------------
   procedure Unload_Parallel_Port;

    Control_Read_Only, Control_Write_Only : exception;
    No_Port_Initialized : exception;
    Portio_Exception    : exception;
    -- see below for uses of these

    ------------------------------------
    -- procedures Make_Control_Pins_[Read,Write]
    --
    -- sets whether the control pins (1,14,16,17)
    -- will be read or written
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    -- 
    -- Control pins start as writeable
    ------------------------------------
    procedure Make_Control_Pins_Write;
    procedure Make_Control_Pins_Read;

    ------------------------------------
    -- procedure Set_Pin
    --
    -- set the value of a Writeable Pin to
    -- true (1) or false (0)
    --
    -- will raise Control_Read_Only if
    -- attempting to write control pins
    -- without first calling Make_Control_Pins_Write
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    ------------------------------------
    procedure Set_Pin(
       Pin_Number : in Writeable_Pin;
       Value      : in Boolean);


    ------------------------------------
    -- procedure Read_Pin
    --
    -- Read the value of a Readable Pin 
    -- true (1) or false (0)
    --
    -- will raise Control_Write_Only if
    -- attempting to read control pins
    -- without first calling Make_Control_Pins_Read
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    ------------------------------------
    procedure Read_Pin(
       Pin_Number : in Readable_Pin;
       Value      : out Boolean);

    ------------------------------------
    -- procedure Write_Data_Byte
    --
    -- sends value as 8 bits to PINS 2-9
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    ------------------------------------
    procedure Write_Data_Byte(
       Value : in Byte);

    ------------------------------------
    -- procedure Write_Control_Nibble
    --
    -- sends value as 4 bits to PINS 1,14,16,17
    -- PIN 17 is most significant
    --
    -- will raise Control_Read_Only if
    -- attempting to write control pins
    -- without first calling Make_Control_Pins_Write
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    ------------------------------------
    procedure Write_Control_Nibble(
       Value : in Nibble);

    ------------------------------------
    -- procedure Read_Data_Nibble
    --
    -- will read 4 bits from PINS 11,12,13,15
    -- into a byte (PIN 15 is most significant)
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    ------------------------------------
    procedure Read_Data_Nibble(
       Value : out Nibble);

    ------------------------------------
    -- procedure Read_Control_Nibble
    --
    -- reads value of 4 bits from PINS 1,14,16,17
    -- PIN 17 is most significant
    --
    -- will raise Control_Write_Only if
    -- attempting to write control pins
    -- without first calling Make_Control_Pins_Write
    --
    -- raises No_Port_Initialized if
    -- Initialize_Parallel_Port not called
    ------------------------------------
    procedure Read_Control_Nibble(
       Value : out Nibble);
end Mcc.Parallel;